"use strict";

this.name = "spicy_hermits_relocator";

this.startUp = function() {
	if (missionVariables["spicy_hermits"]) {
		this.$movedHermits = JSON.parse(missionVariables["spicy_hermits"]);
	}
	else this.$movedHermits = new Object();
}

//save relocated hermits
this.playerWillSaveGame = function() {
	if (this.$movedHermits) {
		missionVariables["spicy_hermits"] = JSON.stringify(this.$movedHermits);
	}
}

this.systemWillPopulate  = function() {
	var currentSystem = this._getCurrentSystemData();
	if (currentSystem) {
		//create as many populators as needed.
		for (var i = 0; i < currentSystem.length; i++) {
			system.setPopulator("spicy_hermits" + i, {
				callback: function(pos) {

					//i like to move it, move it
					var origPosition = this._toVect(this._getCurrentSystemData()[this.$hermitPointer][0]);
					var offset = pos.subtract(origPosition);
					
					//since we only know the position and we need an entity to focus on, let's create a dummy one.
					var positionEntity = system.addVisualEffect("spicy_hermits_dummy", origPosition);
					var hermitage = system.entitiesWithScanClass("CLASS_ROCK", positionEntity, 1E3);
					for (var i = 0; i < hermitage.length; i++) {						
						//identify, move  and tag the hermit
						if (hermitage[i].isStation) {
							var hermit = hermitage[i];
							hermit.position = hermit.position.add(offset);
							hermit.spicy_hermits_original_position = origPosition;
							//add an asteroid field, using core rule
							var size = 1 + Math.floor(system.scrambledPseudoRandomNumber(Math.floor(pos.x)) * 11);
							var rocks = system.addShips("asteroid", size, hermit.position, 25E3);
						}
					}
					
					//remove the position marker and advance the hermit pointer
					positionEntity.remove();
					this.$hermitPointer++;
					
				}.bind(this),
				location: "LANE_WPS",
				locationSeed: currentSystem[i][1]
			})
		}
	}
	this.$hermitPointer = 0;
}

//public function for hermit AIs to report death.
this._reportDeath = function(origPosition) {
	
	//create a new location seed
	var newSeed = 10000 + Math.floor(Math.random() * 90000);
	var dataElement = [origPosition, newSeed];
	
	//get and if needed init current system stored data
	var currentSystemData = this._getAndInitCurrentSystemData();
	
	//see if there's already an entry for this hermit
	//if yes, update entry
	var newHermit = true;
	for (var i = 0; i < currentSystemData.length; i++) {
		if (this._toVect(currentSystemData[i][0]).subtract(origPosition).magnitude() === 0) {
			currentSystemData[i] = dataElement;
			newHermit = false;
			break;
		}
	}
	//if not, create a new entry
	if (newHermit) currentSystemData.push(dataElement);
	
}

//it seems that position vector gets converted to a generic object on save/load. this converts it back to vector.
this._toVect = function(storedObj) {
	var vect = new Vector3D();
	vect.x = storedObj.x;
	vect.y = storedObj.y;
	vect.z = storedObj.z;
	return vect;
}

//get current system stored data
this._getCurrentSystemData = function() {
	if (!this.$movedHermits[galaxyNumber]) return([]);
	if (!this.$movedHermits[galaxyNumber][system.ID]) return([]);
	return this.$movedHermits[galaxyNumber][system.ID];
}

//get current system stored data and init new entries
this._getAndInitCurrentSystemData = function() {
	if (!this.$movedHermits[galaxyNumber]) {
		 this.$movedHermits[galaxyNumber] = new Object();
	}
	if (!this.$movedHermits[galaxyNumber][system.ID]) {
		this.$movedHermits[galaxyNumber][system.ID] = new Array();
	}
	return this.$movedHermits[galaxyNumber][system.ID];
}
